Memory Access Microbenchmark: Clojure, F#, Go, Julia, Lua, Racket, Ruby, Self
togototo.wordpress.com
togototo.wordpress.com
Anyway, this problem should not be solved this way in python either way. I would have stored everything in a Pandas table and computed stuff from there.
I understand that a Pandas table would be a better idea, but the purpose of this benchmark is comparing the speed of the raw languages.
I think the strength of the languages have in part to do with their libraries. You have to pick the right tools for the right task.
The pandas version runs 3x faster than regular python, but 17x slower than C.
*Edit: I tried your faster_py.py, but it didn't seem any faster than Pypy2.py in the repo (running both with Pypy). I haven't yet got the Pandas version to work due to library compatibility issues (I've got about five versions of Python installed; still working on it).
In many cases, pypy cannot be used, because it's not compatible with all the libraries.
https://gist.github.com/mattchamb/20019b22ae841ff5ce1b
Im not sure if it is any faster though; as each run varies quite widely from the previous one
tradesArray.[i] <- {
TradeId = idx;
ClientId = idx;
...
}would lead to the creation of a new object every time whereas:
trades.[i].TradeId <- (int64 i)
trades.[i].ClientId <- (int64 1) etc.
doesn't.In my edited one, the trade object is immutable, so you could just reuse the same object without editing it. In FS.fs, a whole new set of objects is created/allocated on the heap for each test iteration; whereas in FS2.fs, pre-allocated memory is just being edited in place. The first approach seems much more likely to create GC pressure than the second approach.
Similarly, there are a few differences between the implementation in the different languages. For example, in CS.cs, the entire trades array is being re-allocated in each iteration of the test, whereas in the F# tests, it is allocated up front. Changing the array allocation to once only in the C# implementation caused a large speedup (from about 5 secs down to about 1.5 secs on my machine).
Speaking of the C# implementation, here is a slightly cleaned up version with the altered array allocation: https://gist.github.com/mattchamb/9152487
As a side note, I changed it from using DateTime.Now to using the Stopwatch class. The reason for this is that DateTime.Now has limited timer resolution (about 10ms), which you can see in the remarks section of the msdn docs (http://msdn.microsoft.com/en-us/library/system.datetime.now(...)
Edit: yeah, pretty much ignore me, the array initialisation for implementation 1 of F# and C# is almost incomparable to that of the C implementation; which is why the first implementations are so much slower.
If you wanted them to be more directly comparable then for C, you should have an array of pointers which you then fill with pointers to malloc'd addresses.
I should have noticed that sooner... i guess this is why you dont read code at 1am.
> Enables garbage collection that is more conservative in reclaiming objects. Full collections occur only if the system is under memory pressure, whereas generation 0 and generation 1 collections might occur more frequently ... This mode is not available for the server garbage collector.
Just for fun, you wouldn't want to run in LowLatency for too long.
[1] http://msdn.microsoft.com/en-us/library/system.runtime.gcset...
[2] http://msdn.microsoft.com/en-us/library/system.runtime.gclat...